<?php

/**
 * Ajax handler for updating form for creating fields.
 */
function quick_data_update_field_form($form, $form_state) {
  return $form['settings'];
}

/**
 * Quick data form routeur.
 */
function quick_data_add_field_form($form, &$form_state) {

  if (!isset($form_state['stage'])) {
    $form_state['stage'] = 'quick_data_collect_field_label';
  }

  switch ($form_state['stage']) {
    case 'quick_data_collect_field_label':
      return quick_data_collect_field_label($form, $form_state);
      break;

    case 'quick_data_create_field_data':
      return quick_data_create_field_data($form, $form_state);
      break;

    default:
      return quick_data_collect_field_label($form, $form_state);
      break;
  }
}

/*
 * Quick data all step submit
 */
function quick_data_add_field_form_submit($form, &$form_state) {
  $values = isset($form_state['multistep_values']['quick_data_collect_field_label']) ? $form_state['multistep_values']['quick_data_collect_field_label'] : [];
  switch ($form_state['stage']) {
    case 'quick_data_create_field_data':
      $form_state['multistep_values'][$form_state['stage']] = $form_state['values'];
      if ($form_state['triggering_element']['#value'] != 'Back') {
        quick_data_create_field_data_submit($form, $form_state);
        $form_state['complete'] = TRUE;
      }
      break;

    default:
      $form_state['multistep_values'][$form_state['stage']] = $form_state['values'];
      $form_state['new_stage'] = quick_data_add_field_move_to_next_stage($form, $form_state);
      break;
  }

  if (isset($form_state['complete'])) {
    if (empty($values['bundle'])) {
      $values['bundle'] = strtolower(str_replace(' ', '_', _remove_accent($values['name'])));
    }
    switch ($values['type']) {
      case 'node':
        $goto = 'admin/structure/types/manage/' . $values['bundle'] . '/fields';
        break;
      case 'user':
        $goto = 'admin/config/people/accounts/fields';
        break;
      case 'taxonomy_term':
        $goto = 'admin/structure/taxonomy/' . $values['bundle'] . '/fields';
        break;
      case 'field_collection_item':
        $goto = 'admin/structure/field-collections/' . $values['bundle'];
        break;
      case 'paragraphs_item':
        $goto = 'admin/structure/paragraphs/' . $values['bundle'] . '/fields';
        break;
      case 'data':
        $goto = 'admin/structure/data/edit/' . $values['bundle'];
        break;
      case 'data_custom':
        $goto = 'admin/structure/views/view/' . $values['bundle'];
        break;
      default:
        $goto = 'admin/structure/fields-quick-data';
    }
    drupal_goto($goto);
  }

  if ($form_state['triggering_element']['#value'] == 'Back') {
    $form_state['new_stage'] = quick_data_add_field_move_to_previous_stage($form, $form_state);
  }

  if (isset($form_state['multistep_values']['form_build_id'])) {
    $form_state['values']['form_build_id'] = $form_state['multistep_values']['form_build_id'];
  }
  $form_state['multistep_values']['form_build_id'] = $form_state['values']['form_build_id'];
  $form_state['stage'] = $form_state['new_stage'];
  $form_state['rebuild'] = TRUE;
}

/**
 * Given the current stage the user is on, calculate what the next step would be
 *
 * @param type $form_state
 *
 * @return string
 */
function quick_data_add_field_move_to_next_stage($form, &$form_state) {
  switch ($form_state['stage']) {
    case 'quick_data_collect_field_label':
      return 'quick_data_create_field_data';
      break;
    case 'quick_data_create_field_data':
      return 'quick_data_collect_field_label';
      break;
  }
}

/**
 * Given the current stage the user is on, calculate what the previous step
 * would be
 *
 * @param type $form_state
 *
 * @return string
 */
function quick_data_add_field_move_to_previous_stage($form, &$form_state) {
  switch ($form_state['stage']) {
    case 'quick_data_create_field_data':
      return 'quick_data_collect_field_label';
      break;
  }

}

/*
 * Quick data form 1st step
 */
function quick_data_collect_field_label($form, &$form_state) {

  $values = isset($form_state['multistep_values']['quick_data_collect_field_label']) ? $form_state['multistep_values']['quick_data_collect_field_label'] : [];
  $option = [
    'user' => t('User'),
    'node' => t('Node'),
    'taxonomy_term' => t('Taxonomy'),
    'data_custom' => t('Custom table'),
  ];
  if (module_exists('field_collection')) {
    $option['field_collection_item'] = t('Field collection');
  }
  if (module_exists('paragraphs')) {
    $option['paragraphs_item'] = t('Paragraphs');
  }
  if (module_exists('data')) {
    $option['data'] = t('Data');
  }
  $id_wrapper = 'quick-data-add-field-bundle';
  $form['collect_field_label']['type'] = [
    '#type' => 'select',
    '#title' => t('Type'),
    '#options' => $option,
    '#default_value' => isset($values['type']) ? $values['type'] : 'user',
    '#description' => t('Set your Type'),
    '#ajax' => [
      'callback' => 'ajax_quick_data_collect_field_callback',
      'wrapper' => $id_wrapper,
      'method' => 'replace',
    ],
  ];
  $type = !empty($form_state['values']['type']) ? $form_state['values']['type'] : NULL;
  if (empty($type) && !empty($values['type'])) {
    $type = $values['type'];
  }
  $form['collect_field_label']['bundle'] = [
    '#type' => 'select',
    '#title' => t('Bundle'),
    '#empty_option' => t('- New bundle -'),
    '#options' => _collect_field_label_type($type),
    '#default_value' => isset($values['bundle']) ? $values['bundle'] : NULL,
    '#description' => t('Set your bundle'),
    '#states' => [
      'invisible' => [
        ':input[name="type"]' => ['value' => 'user'],
      ],
    ],
    '#prefix' => "<div id='$id_wrapper'>",
    '#suffix' => '</div>',
  ];
  $form['collect_field_label']['name'] = [
    '#type' => 'textfield',
    '#title' => t('Machine name'),
    '#maxlength' => 32,
    '#default_value' => isset($values['name']) ? $values['name'] : NULL,
    '#description' => t('Set your machine name, if new bundle. Set your table name if you want create new table'),
    '#states' => [
      'invisible' => [
        ':input[name="type"]' => ['value' => 'field_collection_item'],
      ],
      'visible' => [
        ':input[name="bundle"]' => ['value' => ''],
      ],
    ],
  ];
  if (!empty($type) && $type == 'data_custom') {
    $form['collect_field_label']['name']['#title'] = t('Table name');
    $form['collect_field_label']['name']['#description'] = t('Set your table name');
  }
  $form['collect_field_label']['field_exist'] = [
    '#type' => 'checkbox',
    '#title' => t('Exist field'),
    '#default_value' => isset($values['field_exist']) ? $values['field_exist'] : NULL,
    '#description' => t('If you want load exits field'),
    '#ajax' => [
      'callback' => 'ajax_quick_data_collect_label_callback',
      'wrapper' => 'quick-data-add-field-label',
      'method' => 'replace',
    ],
  ];

  $form['collect_field_label']['field_label'] = [
    '#type' => 'textarea',
    '#title' => t('Field label'),
    '#description' => t('Enter your list field label'),
    '#default_value' => isset($values['field_label']) ? $values['field_label'] : NULL,
    '#prefix' => '<div id="quick-data-add-field-label">',
    '#suffix' => '</div>',
  ];
  if (empty($form['collect_field_label']['field_label']['#default_value'])) {
    if (!empty($form_state["values"]["field_exist"])) {
      $form['collect_field_label']['field_label']['#value'] = _load_type_bundle_fields_info($form_state["values"]["type"], $form_state["values"]["bundle"]);
    }
    elseif (isset($form['collect_field_label']['field_label']['#value'])) {
      unset($form['collect_field_label']['field_label']['#value']);
    }
  }
  $generate_module = [
    'search_api' => FALSE,
    'feeds' => FALSE,
    'views' => FALSE,
  ];
  $module_names = [];
  foreach ($generate_module as $module => &$existe) {
    if (module_exists($module)) {
      $existe = TRUE;
      $info = system_get_info('module', $module);
      $module_names[$module] = $info['name'];
    }
  }
  $form['collect_field_label']['generate'] = [
    '#type' => 'fieldset',
    '#title' => t('Extra generate'),
    '#collapsible' => TRUE, // Added
    '#collapsed' => FALSE,  // Added
    '#description' => t('Gererate with modules ') . implode(', ', $module_names),
  ];
  if (!empty($module_names)) {

    foreach ($module_names as $module => $module_name) {
      $form['collect_field_label']['generate'][$module] = [
        '#type' => 'checkbox',
        '#title' => $module_name,
        '#default_value' => isset($values['generate'][$module]) ? $values['generate'][$module] : 0,
      ];
      if ($module == 'search_api') {
        $servers_search_api = search_api_server_load_multiple(FALSE);
        $optionServerSearch = [];
        if (!empty($servers_search_api)) {
          foreach ($servers_search_api as $server_search_api => $search_api) {
            $optionServerSearch[$search_api->machine_name] = $search_api->name;
          }
        }
        $form['collect_field_label']['generate']['SearchApiServer'] = [
          '#type' => 'select',
          '#title' => t('Search Server'),
          '#empty_option' => t('- New Search Server -'),
          '#description' => t('Must have server like ') . l('Database Search', 'https://www.drupal.org/project/search_api_db') . t(' or ') . l('Elasticsearch', 'https://www.drupal.org/project/search_api_elasticsearch'),
          '#options' => $optionServerSearch,
          '#default_value' => isset($values['generate']['SearchApiServer']) ? $values['generate']['SearchApiServer'] : NULL,
          '#states' => [
            'visible' => [
              "input[name='search_api']" => ["checked" => TRUE],
            ],
          ],
        ];
      }
      // paragraphs_item don't support feeds
      if ($module == 'feeds') {
        $form['collect_field_label']['generate'][$module]['#states'] = [
          'invisible' => [
            [':input[name="type"]' => ['value' => 'paragraphs_item']],
            'or',
            [':input[name="type"]' => ['value' => 'data']],
            'or',
            [':input[name="type"]' => ['value' => 'data_custom']],
          ],
        ];
      }
      if ($module == 'search_api') {
        $form['collect_field_label']['generate'][$module]['#states'] = [
          'invisible' => [
            [':input[name="type"]' => ['value' => 'data']],
            'or',
            [':input[name="type"]' => ['value' => 'data_custom']],
          ],
        ];
      }
      if ($module == 'views') {
        $form['collect_field_label']['generate'][$module]['#default_value'] = isset($values['generate'][$module]) ? $values['generate'][$module] : 1;
      }
    }
  }
  $form['next'] = [
    '#type' => 'submit',
    '#value' => t('Next'),
  ];
  return $form;
}

// Ajax Call
function ajax_quick_data_collect_field_callback(&$form, &$form_state) {
  return $form['collect_field_label']['bundle'];
}

function ajax_quick_data_collect_label_callback(&$form, &$form_state) {
  return $form['collect_field_label']['field_label'];
}

function _load_type_bundle_fields_info($type = NULL, $bundle = NULL) {
  if (is_null($type) && is_null($bundle)) {
    return NULL;
  }
  $params = [
    'entity_type' => $type,
  ];
  if (!empty($bundle)) {
    $params ['bundle'] = $bundle;
  }
  $fields = field_read_instances($params);
  foreach ($fields as $field) {
    $field_info = field_info_field($field['field_name']);
    $infos[] = [
      'title' => $field['field_name'] . '|' . $field['label'] . '|' . $field_info["type"] . '|' . $field["widget"]["type"],
      'weight' => $field["widget"]["weight"],
    ];
  }
  uasort($infos, 'drupal_sort_weight');
  foreach ($infos as $info) {
    $out[] = $info['title'];
  }
  return implode("\n", $out);
}

/*
 * get all type of bundle
 */
function _collect_field_label_type($bundle = FALSE) {
  $type = [];
  if (!empty($bundle) && !in_array($bundle, ['user', 'data'])) {
    $entity_type = entity_get_info($bundle);
    if (!empty($entity_type['bundles'])) {
      foreach ($entity_type['bundles'] as $key => $name) {
        $type[$key] = $name['label'];
      }
    }
  }
  return $type;
}

/*
 * return machine name of bundle
 */
function _collect_field_machine_names_type($bundle) {
  if (!empty($bundle) && $bundle != 'user') {
    $entity_type = entity_get_info($bundle);
    if (!empty($entity_type['bundles'])) {
      return array_keys($entity_type['bundles']);
    }
  }
  return [];
}

/*
 * Quick data form step 2
 */
function quick_data_create_field_data($form, &$form_state) {

  $form['#value'] = $form_state['values'];
  $values = isset($form_state['multistep_values']['quick_data_create_field_data']) ? $form_state['multistep_values']['quick_data_create_field_data'] : [];

  $form['back'] = [
    '#type' => 'submit',
    '#value' => t('Back'),
  ];

  $form['next'] = [
    '#type' => 'submit',
    '#value' => t('Submit'),
  ];

  return $form;
}

/*
 * Quick data form on submit
 */
function quick_data_create_field_data_submit($form, &$form_state) {
  $values = !empty($form_state['multistep_values']['quick_data_collect_field_label']) ? $form_state['multistep_values']['quick_data_collect_field_label'] : FALSE;
  if (!empty($form_state['input']['fields'])) {
    $bundle = $values['bundle'];
    $name = ucfirst(str_replace('_', ' ', $bundle));
    if (empty($bundle)) {
      $name = _convert_machine_name($values['name']);
    }
    $entity_type = $values['type'];
    $new_name = $values['name'];
    // Define the node type.
    switch ($entity_type) {
      case 'node' :
        $feeds_processor = [
          'plugin_key' => 'FeedsNodeProcessor',
          'config' => [
            'expire' => '-1',
            'author' => 0,
            'authorize' => 0,
          ],
        ];
        break;
      case 'taxonomy_term' :
        $vocabulary = taxonomy_vocabulary_machine_name_load($name);
        if (!$vocabulary) {
          $vocabulary = new stdClass();
          $vocabulary->name = $new_name;
          $vocabulary->machine_name = $name;
          $vocabulary->description = t('Create a new taxonomy ') . ' ' . $new_name;
          $vocabulary->module = 'taxonomy';
          taxonomy_vocabulary_save($vocabulary);
        }
        $feeds_processor = [
          'plugin_key' => 'FeedsTermProcessor',
          'config' => [
            'vocabulary' => $vocabulary->vid,
            'bundle' => !empty($name) ? $name : $bundle,
          ],
        ];
        break;
      case 'paragraphs_item' :
        $feeds_processor = FALSE;// not support paragraphs
        break;
      case 'user' :
        $feeds_processor = [
          'plugin_key' => 'FeedsUserProcessor',
          'config' => [
            'bundle' => NULL,
            'roles' => [],
            'status' => 1,
            'defuse_mail' => FALSE,
          ],
        ];
        break;
      case 'field_collection_item' :
        $feeds_processor = [
          'plugin_key' => 'FeedsFieldCollectionProcessor',
          'config' => [
            'field_name' => $bundle,
            'host_entity_type' => $entity_type,
            'host_entity_importer_id' => '',
            'host_entity_importer_nid' => NULL,
            'guid_field_name' => '',
            'is_field' => 1,
          ],
        ];
        break;
      case 'data_custom' :
      case 'data' :
        break;
    }
    $get_instance = field_info_instances($entity_type, $bundle);

    if (!empty($entity_type) && !empty($new_name) && empty($get_instance[$new_name])) {
      $bundle = strtolower(str_replace(' ', '_', _remove_accent($new_name)));
      $name = ucfirst($new_name);
      if ($entity_type == 'node') {
        $new_content_type = [
          'type' => $bundle,
          'name' => $name,
          'description' => t('Create a new content ') . ' ' . $name,
          'title_label' => t('Title'),
          'base' => 'node_content',
          'custom' => TRUE,
        ];
        $node_type = node_type_set_defaults($new_content_type);
        // Save the content type
        node_type_save($node_type);
      }
      if ($entity_type == 'paragraphs_item') {
        $bundle_paragraphs = new stdClass();
        $bundle_paragraphs->bundle = $bundle;
        $bundle_paragraphs->locked = TRUE;
        $bundle_paragraphs->name = $name;
        paragraphs_bundle_save($bundle_paragraphs);
      }
      if ($entity_type == 'data_custom') {
        $schema['description'] = $new_name;
        foreach ($form_state['input']['fields'] as $field_info) {
          $schema['fields'][$field_info['field_name']] = _build_field_definition($field_info);
        }
        try {
          db_create_table($bundle, $schema);
          drupal_set_message(t('Create new table ') . "<strong>$bundle</strong>");
        } catch (DatabaseSchemaObjectExistsException $e) {
          drupal_set_message(t('Error creating table.'), 'error');
        }
      }
      if ($entity_type == 'data') {
        foreach ($form_state['input']['fields'] as $field_info) {
          $field = [
            'name' => $field_info['field_name'],
            'type' => $field_info['type'],
            'size' => $field_info['widget_type'],
          ];
          $schema['fields'][$field_info['field_name']] = data_build_field_definition($field);
          $meta['fields'][$field_info['field_name']]['label'] = check_plain($field_info['label']);
        }
        if (!empty($schema)) {
          $bundle_data = data_create_table($bundle, $schema, $new_name);
          $meta = $bundle_data->update(['meta' => $meta]);
          //@todo will remove if finish views && feeds
          drupal_goto('admin/structure/data/edit/' . $bundle);
        }
      }
    }

    if ($entity_type == 'user') {
      $bundle = 'user';
      $name = ucfirst($bundle);
    }

    $type_index = [
      'text' => 'text',
      'text_long' => 'text',
      'field_hidden_text_long' => 'text',
      'field_hidden_text' => 'text',
      'office_hours' => 'text',
      'integer' => 'integer',
      'field_hidden_integer' => 'integer',
      'number_integer' => 'integer',
      'list_integer' => 'integer',
      'entityreference' => 'integer',
      'decimal' => 'decimal',
      'field_hidden_decimal' => 'decimal',
      'list_float' => 'decimal',
      'number_decimal' => 'decimal',
      'number_float' => 'decimal',
      'field_hidden_float' => 'decimal',
      'date' => 'date',
      'datetime' => 'date',
      'datestamp' => 'date',
      'field_timetable_timetable' => 'date',
      'duration' => 'duration',
      'boolean' => 'boolean',
      'list_boolean' => 'boolean',
      'number_boolean' => 'boolean',
      'image' => 'uri',
      'file' => 'uri',
      'media' => 'uri',
      'uri' => 'uri',
    ];
    $type_feed_target = [
      'datetime' => ':start',
      'datestamp' => ':start',
      'date' => ':start',
      'file' => ':uri',
      'double_field' => ':first',
      'addressfield' => ':thoroughfare',
      'name' => ':family',
      'phone' => ':url',
    ];

    module_invoke_all('quick_data_feed_target', $type_feed_target);
    $fields_index['search_api_language'] = ['type' => 'string'];
    foreach ($form_state['input']['fields'] as $field_name_complet => $field_info) {
      //@todo get field prefix
      $field_prefix = 'field_';
      $field_name = $field_prefix . $field_info['field_name'];
      // Create the field base.
      if (!field_info_field($field_name) && !in_array($entity_type, [
          'data',
          'data_custom',
        ])) {
        $field = [
          'field_name' => $field_name,
          'type' => $field_info['type'],
        ];
        field_create_field($field);
      }
      $target = !empty($type_feed_target[$field_info['type']]) ? $type_feed_target[$field_info['type']] : '';
      $feeds_mappings[] = [
        'source' => $field_info['label'],
        'target' => $field_name . $target,
        'unique' => FALSE,
      ];
      $string_type_index = !empty($type_index[$field_info['type']]) ? $type_index[$field_info['type']] : 'string';
      $fields_index[$field_name] = ['type' => $string_type_index];
      $fields_view_index[$field_name] = [
        'id' => $field_name,
        'field' => $field_name,
        'table' => 'search_api_index_' . $bundle,
      ];
      $fields_view[$field_name] = [
        'id' => $field_name,
        'field' => $field_name,
        'table' => 'field_data_' . $field_name,
        'info' => $field_info['field_name'],
      ];

      // Create the field instance on the bundle.
      $instance = [
        'field_name' => $field_name,
        'entity_type' => $entity_type,
        'label' => $field_info['label'],
        'bundle' => $bundle,
        'widget' => [
          'type' => $field_info['widget_type'],
        ],
      ];
      $test_instance_exist = field_info_instance($entity_type, $field_name, $bundle);
      if (!$test_instance_exist && !in_array($entity_type, [
          'data',
          'data_custom',
        ])) {
        field_create_instance($instance);
      }
    }
  }

  if (!empty($values['search_api'])) {
    if (empty($values['SearchApiServer'])) {
      $search_api_class = FALSE;
      if (module_exists('search_api_elasticsearch_elastica')) {
        $search_api_class = 'search_api_elasticsearch_elastica_service';
      }
      if (module_exists('search_api_db')) {
        $search_api_class = 'search_api_db_service';
      }
      if ($search_api_class) {
        $values_server = [
          'name' => 'Default db server',
          'machine_name' => 'default_db_server',
          'enable' => TRUE,
          'class' => $search_api_class,
        ];
        $server = entity_create('search_api_server', $values_server);
        drupal_set_message(t('Create new search server ') . l($values_server['name'], 'admin/config/search/search_api/server/' . $values_server['machine_name'] . '/edit'));
      }
    }
    else {
      $server = entity_load('search_api_server', [$values['SearchApiServer']]);
      $server = end($server);
    }
    if (!empty($server)) {
      $options = [
        'database' => 'default:default',
        'min_chars' => 3,
        'partial_matches' => 1,
      ];
      $server->configurationFormSubmit([], $options, $form_state);
      $server->save();
      $bundles = ['bundles' => [$bundle]];
      $options = ['index_directly' => 1, 'cron_limit' => 50];
      $values_bundles = [
        'name' => $name,
        'machine_name' => $bundle,
        'item_type' => $entity_type,
        'options' => $options,
        'server' => $server->machine_name,
        'read_only' => FALSE,
        'enable' => TRUE,
        'old_type' => $entity_type,
        'datasource_config' => FALSE,
      ];
      $index = search_api_index_load($bundle);
      if (!$index) {
        $search_api_index = search_api_index_insert($values_bundles);
        $index = search_api_index_load($search_api_index);
      }
      $options['datasource'] = $bundles;
      $options['fields'] = $fields_index;
      $index->update(['options' => $options]);
      drupal_set_message(t('Create new search index  ') . l($name, 'admin/config/search/search_api/index/' . $bundle . '/edit'));
    }
    else {
      drupal_set_message(t('Cannot create new search server, please install ') . l('Search API Database Search', 'https://www.drupal.org/project/search_api_db'));
    }
  }
  if (!empty($values['views'])) {
    $base_table = $bundle;
    if (!empty($values['search_api'])) {
      $base_table = 'search_api_index_' . $bundle;
    }
    $view = views_get_view($bundle);
    if (empty($view)) {
      $view = views_new_view();
      $view->name = $bundle;
    }
    else {
      $view->name = $bundle . rand(0, 99);
    }
    $view->tag = 'default';
    $view->base_table = $entity_type;
    if ($entity_type == 'data_custom') {
      $view->base_table = $base_table;
    }
    if (!empty($values['search_api'])) {
      $view->base_table = $base_table;
    }
    if ($entity_type == "taxonomy_term") {
      $view->base_table = 'taxonomy_term_data';
    }
    if ($entity_type == "user") {
      $view->base_table = 'users';
    }
    $view->human_name = ucfirst($name);
    $view->core = 7;
    $view->api_version = '3.0';
    $view->disabled = FALSE; /* Edit this to true to make a default view disabled initially */
    /* Display: Master */
    $handler = $view->new_display('default', 'Master', 'default');
    $handler->display->display_options = [
      'title' => ucfirst($name),
      'use_more_always' => FALSE,
      'access' => ['type' => 'none'],
      'cache' => ['type' => 'none'],
      'query' => ['type' => 'views_query'],
      'exposed_form' => ['type' => 'basic'],
      'pager' => ['type' => 'full', 'options' => ['items_per_page' => 20]],
      'style_plugin' => 'table',
    ];

    if ($entity_type == "user") {
      $handler->display->display_options['access']['type'] = 'perm';
      $handler->display->display_options['access']['perm'] = 'access user profiles';
    }

    if (!empty($values['search_api'])) {
      $handler->display->display_options['fields'] = $fields_view_index;
    }
    else {
      switch ($entity_type) {
        case "node":
          if (!empty($fields_view)) {
            $handler->display->display_options['fields'] = $fields_view;
          }
          /* Champ: Content: Title */
          $handler->display->display_options['fields']['title'] = [
            'id' => 'title',
            'table' => 'node',
            'field' => 'title',
          ];
          /* Sort criterion: Content: Post date */
          $handler->display->display_options['sorts']['created'] = [
            'id' => 'created',
            'table' => 'node',
            'field' => 'created',
            'order' => 'DESC',
          ];
          /* Filter criterion: Content: Public */
          $handler->display->display_options['filters']['status'] = [
            'id' => 'status',
            'table' => 'node',
            'field' => 'status',
            'value' => 1,
            'group' => 1,
            'operator' => FALSE,
          ];
          /* Filter criterion: Content: Type */
          $handler->display->display_options['filters']['type'] = [
            'id' => 'type',
            'table' => 'node',
            'field' => 'type',
            'value' => [$bundle => $bundle],
          ];
          break;
        case "user":

          if (!empty($fields_view)) {
            $handler->display->display_options['fields'] = $fields_view;
          }
          /* Champ: User: name */
          $handler->display->display_options['fields']['name'] = [
            'id' => 'name',
            'table' => 'users',
            'field' => 'name',
          ];

          /* Sort criterion: Utilisateur: Nom */
          $handler->display->display_options['sorts']['name'] = [
            'id' => 'name',
            'table' => 'users',
            'field' => 'name',
          ];

          /* Filter criterion: User: Active */
          $handler->display->display_options['filters']['status'] = [
            'id' => 'status',
            'table' => 'users',
            'field' => 'status',
            'value' => '1',
          ];
          break;
        case "taxonomy_term":

          if (!empty($fields_view)) {
            $handler->display->display_options['fields'] = $fields_view;
          }
          /* Champ: Term taxonomy: Name */
          $handler->display->display_options['fields']['name'] = [
            'id' => 'name',
            'table' => 'taxonomy_term_data',
            'field' => 'name',
            'link_to_taxonomy' => TRUE,
          ];
          $vid = taxonomy_vocabulary_machine_name_load($bundle);
          /* Filter criterion: Term taxonomy: Vocabulary */
          $handler->display->display_options['filters']['vid'] = [
            'id' => 'vid',
            'table' => 'taxonomy_term_data',
            'field' => 'vid',
            'value' => [$vid => $vid],
          ];

          break;
        case "field_collection_item":

          /* Relationship: Field collection item: Entity with the field */
          //check entity reference
          $set_entity_type = 'node';
          foreach (['user', 'taxonomy_term'] as $get_entity_type) {
            $entities = field_info_instances($get_entity_type);
            $flag = FALSE;
            if (!empty($entities)) {
              foreach ($entities as $entity_check) {
                if (!empty($entity_check[$bundle])) {
                  $set_entity_type = $entity_check[$bundle]["entity_type"];
                  break;
                }
              }
            }
            if ($flag) {
              break;
            }
          }
          $handler->display->display_options['relationships'] = [
            $bundle . '_' . $set_entity_type => [
              'id' => $bundle . '_' . $set_entity_type,
              'table' => 'field_collection_item',
              'field' => $bundle . '_' . $set_entity_type,
              'required' => TRUE,
            ],
          ];

          /* Champ: Field collection item: Field collection item ID */
          $handler->display->display_options['fields']['item_id'] = [
            'id' => 'item_id',
            'label' => 'Id',
            'table' => 'field_collection_item',
            'field' => 'item_id',
            'separator' => '',
          ];
          foreach ($fields_view as $field_name_view => $field_value_view) {
            $handler->display->display_options['fields'][$field_name_view] = [
              'id' => $field_name_view,
              'table' => 'field_data_' . $field_value_view['field'],
              'field' => $field_value_view['field'],
            ];
          }
          /* Field: Field collection item: Edit Field Collection */
          $handler->display->display_options['fields']['edit_link'] = [
            'id' => 'edit_link',
            'table' => 'field_collection_item',
            'field' => 'edit_link',
            'label' => t('edit'),
          ];
          /* Field: Field collection item: Add Field Collection */
          $handler->display->display_options['fields']['delete_link'] = [
            'id' => 'delete_link',
            'table' => 'field_collection_item',
            'field' => 'delete_link',
            'label' => t('delete'),
          ];
          /* Footer: Field collection item: Add Field Collection */
          $handler->display->display_options['footer']['add_link'] = [
            'id' => 'add_link',
            'table' => 'field_collection_item',
            'field' => 'add_link',
            'label' => t('Add'),
          ];
          break;
        case 'paragraphs_item':
          break;
        case 'data' :
          //@todo for data tables
          break;
        case 'data_custom' :
          $view->base_table = $base_table;
          if (module_exists('view_custom_table')) {
            $state_form = [
              'values' => ['columns' => ['entity' => '']],
              'table_name' => $bundle,
              'table_description' => $name,
              'redirect' => 'admin/structure/fields-quick-data',
            ];
            module_load_include('inc', 'view_custom_table', 'view_custom_table.admin');
            view_custom_table_add_custom_table_column_relationship_form_submit([], $state_form);

            foreach ($fields_view as $field_value_view) {
              $field_name_view = $field_value_view['info'];
              $handler->display->display_options['fields'][$field_name_view] = [
                'id' => $field_name_view,
                'table' => $base_table,
                'field' => $field_name_view,
              ];
            }
          }
          break;
      }
    }
    /* Display: Page */
    $handler = $view->new_display('page', 'Page', 'page');
    $path = str_replace('_', '-', $bundle);
    if (drupal_valid_path($path) || drupal_lookup_path('source', $path)) {
      try {
        $path .= random_int(1, 10);
      } catch (Exception $e) {
      }
    }
    $handler->display->display_options['path'] = $path;

    $view->execute_display('default', []);
    $view->save();
    drupal_set_message(t('Create new view ') . l($name, 'admin/structure/views/view/' . $view->name . '/edit'));
  }
  if (!empty($values['feeds'])) {
    $importer = feeds_importer($bundle);
    $config = [
      'name' => $name,
      'id' => $bundle,
      'description' => t('Import CSV ') . $name,
      'fetcher' => [
        'plugin_key' => 'FeedsFileFetcher',
        'config' => [
          'allowed_extensions' => 'txt csv tsv xml opml',
          'delete_uploaded_file' => FALSE,
          'direct' => FALSE,
          'directory' => 'public://feeds',
          'allowed_schemes' => ['public'],
        ],
      ],
      'parser' => [
        'plugin_key' => 'FeedsCSVParser',
        'config' => [
          'delimiter' => ',',
          'encoding' => 'UTF-8',
          'no_headers' => 0,
        ],
      ],
      'processor' => [
        'config' => [
          'mappings' => $feeds_mappings,
          'insert_new' => 1,
          'update_existing' => 0,
          'update_non_existent' => 'skip',
          'input_format' => 'plain_text',
          'skip_hash_check' => 0,
          'bundle' => $bundle,
          'language' => LANGUAGE_NONE,
        ],
      ],
      'content_type' => '',
      'update' => 0,
      'import_period' => -1,
      'expire_period' => 3600,
      'import_on_create' => 1,
      'process_in_background' => 0,
    ];
    if (!empty($feeds_processor)) {
      $config['processor'] = $config['processor'] + $feeds_processor;
      $config['processor']['config'] = $config['processor']['config'] + $feeds_processor['config'];

      $importer->setConfig($config);
      foreach (['fetcher', 'parser', 'processor'] as $type) {
        $importer->setPlugin($config[$type]['plugin_key']);
        $importer->$type->setConfig($config[$type]['config']);
      }
      $importer->save();
      drupal_set_message(t('Create new feeds ') . l($name, 'admin/structure/feeds/' . $bundle . '/mapping'));
    }
  }
  return $form;
}

/**
 * Defines configuration form fields.
 *
 * Implements drupal_get_form().
 *
 * @param array $form
 * @param array &$form_state
 *
 * @return array
 *   - the return value of system_settings_form()
 */
function _quick_data_admin_form($form, &$form_state) {
  $form['quick_data_settings_stop_word'] = [
    '#type' => 'textfield',
    '#title' => t('Stop word'),
    '#decription' => t('These words will will remove when quick data create machine name for each field.'),
    '#default_value' => variable_get('quick_data_settings_stop_word', 'le la du de des a au en'),
    '#required' => FALSE,
  ];
  return system_settings_form($form);
}

/*
 * helper convert name to machine_name
 * machine name must be less than 26 characters
 */
function _convert_machine_name($field_label, $maxlen = 26) {
  $field_label = str_replace(["field_", "'", "_"], " ", trim($field_label));
  $field_label = _remove_accent($field_label);
  setlocale(LC_CTYPE, 'en_AU.utf8');
  $machine_name = preg_replace('/[^a-z0-9 ]/i', '', iconv("UTF-8", "US-ASCII//TRANSLIT", $field_label));

  $stop_word = variable_get('quick_data_settings_stop_word');
  if (!empty($stop_word)) {
    $words = explode(' ', strtolower($machine_name));
    if ($words[0] == 'field') {
      unset($words[0]);
    }
    $remove_word = explode(' ', strtolower($stop_word));
    $field_label = array_diff($words, $remove_word);
    $machine_name = implode(' ', $field_label);
  }

  $machine_name = substr($machine_name, 0, $maxlen);
  return $machine_name = str_replace('-', '_', drupal_html_id($machine_name));
}

/*
 * helper remove accent
 */
function _remove_accent($str) {
  $unicode = [
    'a' => 'á|à|ả|ã|ạ|ă|ắ|ặ|ằ|ẳ|ẵ|â|ấ|ầ|ẩ|ẫ|ậ|å|æ|Á|À|Ả|Ã|Ạ|Ă|Ắ|Ặ|Ằ|Ẳ|Ẵ|Â|Ấ|Ầ|Ẩ|Ẫ|Ậ|Å|Æ|@|Д|д|Ą|ą',
    'd' => 'đ|Đ|Ḍ|ḍ|ð|Ď|ď',
    'e' => 'é|è|ẻ|ẽ|ẹ|ê|ế|ề|ể|ễ|ệ|É|È|Ẻ|Ẽ|Ẹ|Ê|Ế|Ề|Ể|Ễ|Ệ|Э|э|Ɛ|ɛ|Σ|ε|€|Ę|ę|Ě|ě',
    'i' => 'í|ì|ỉ|ĩ|ị|ï|Í|Ì|Ỉ|Ĩ|Ị|Ï',
    'o' => 'ó|ò|ỏ|õ|ọ|ô|ố|ồ|ổ|ỗ|ộ|ơ|ớ|ờ|ở|ỡ|ợ|ö|ø|Ó|Ò|Ỏ|Õ|Ọ|Ô|Ố|Ồ|Ổ|Ỗ|Ộ|Ơ|Ớ|Ờ|Ở|Ỡ|Ợ|°|¤|Œ|œ|Ø|Ф|ф|Ю|ю|Ѳ|ѳ',
    'u' => 'ú|ù|ủ|ũ|ụ|ư|ứ|ừ|ử|ữ|ự|û|ü|Ú|Ù|Ủ|Ũ|Ụ|Ư|Ứ|Ừ|Ử|Ữ|Ự|Ü|Û|Ц|ц',
    'y' => 'ý|ỳ|ỷ|ỹ|ỵ|ÿ|Ý|Ỳ|Ỷ|Ỹ|Ỵ|Ÿ|¥|Ч|ч|Ɣ|ɣ|γ',
    '2' => '²',
    'c' => 'ç|Ç|Ć|ć',
    's' => 'š|Š|§|Ṣ|ṣ|Ś|ś|Ş|ş',
    'l' => '£|Ł|ł|Ĺ|ĺ|Ľ|ľ',
    'm' => 'μ',
    'n' => 'И|и|ñ|Ñ|Л|л|П|п|Ń|ń',
    'z' => 'Ž|ž|Ẓ|ẓ|Ź|ź|Ż|ż',
    'b' => 'ß|ẞ|Ъ|ъ|Ы|ы|Б|б|Ѣ|ѣ|þ|Þ',
    'x' => 'Ж|ж',
    'w' => 'Ш|ш|Щ|щ|ŵ|Ŵ',
    'r' => 'Я|я|Γ|Ṛ|ṛ',
    'v' => 'Ѵ|ѵ',
    'g' => 'Ǧ|ǧ',
    'h' => 'Ḥ|ḥ',
    't' => 'Ṭ|ṭ|Ť|ť',
  ];
  foreach ($unicode as $nonUnicode => $uni) {
    $str = trim(preg_replace("/($uni)/i", $nonUnicode, $str));
  }
  return $str;
}

function _field_type_options() {
  $types = array_keys(_widget_type_options());
  return drupal_map_assoc($types);
}

function _widget_type_options() {
  $sizes = [
    'int' => [
      'int',
      'tinyint',
      'smallint',
      'mediumint',
      'bigint',
      'bit',
      'boolean',
      'serial',
    ],
    'float' => ['float', 'decimal', 'double', 'real'],
    'date' => ['date', 'datetime', 'timestamp', 'time', 'year'],
    'varchar' => ['varchar', 'char', 'binary', 'varbinary'],
    'text' => ['text', 'tinytext', 'mediumtext', 'longtext', 'json'],
    'blob' => ['tinyblob', 'mediumblob', 'blob', 'longblob'],
    'enum' => ['set', 'enum'],
    'geometry' => [
      'geometry',
      'point',
      'linestring',
      'polygon',
      'multipoint',
      'multilinestring',
      'multipolygon',
      'geometrycollection',
    ],
  ];
  foreach ($sizes as $key => $val) {
    $sizes[$key] = drupal_map_assoc($val);
  }
  return $sizes;
}

function _build_field_definition($field_info) {
  $field = [
    "field_name" => $field_info["field_name"],
    "type" => $field_info["type"],
    'description' => $field_info["label"],
    'mysql_type' => $field_info["widget_type"],
    'pgsql_type' => strtoupper($field_info["widget_type"]),
  ];
  if ($field_info['type'] == 'int') {
    $field['unsigned'] = empty($field_info['unsigned']) ? FALSE : TRUE;
  }
  if ($field_info['type'] == 'varchar') {
    $field['length'] = 255;
  }
  return $field;
}
