<?php

/**
 * Quick data form routeur.
 */
function quick_data_import_form($form, &$form_state, $type = '', $bundle = '', $field = '', $entity_id = '') {

  if (!isset($form_state['stage'])) {
    $form_state['stage'] = 'quick_data_import_load_csv';
  }
  $form_state['storage']['type'] = $type;
  $form_state['storage']['bundle'] = $bundle;
  $form_state['storage']['field'] = $field;
  $form_state['storage']['entity_id'] = $entity_id;
  switch ($form_state['stage']) {
    case 'quick_data_import_field_data':
      return quick_data_import_field_data($form, $form_state);
      break;
    default:
      return quick_data_import_load_csv($form, $form_state);
      break;
  }
}

function quick_data_import_load_csv($form, &$form_state) {
  // Display page 2 if $form_state['storage']['page_two'] is set
  if (isset($form_state['storage']['page_two'])) {
    return quick_data_import_process_csv($form, $form_state);
  }
  $type = $form_state['storage']['type'];
  $bundle = $form_state['storage']['bundle'];
  $field = $form_state['storage']['field'];
  $entity_id = $form_state['storage']['entity_id'];
  //page one load csv
  if (!in_array($type, [
    "node",
    "user",
    "taxonomy",
    "data",
    "paragraphs_item",
    "field_collection_item",
  ])) {
    drupal_set_message(t("Your type %type is not available", ['%type' => $bundle]), 'error');
    return $form;
  }
  //check bundle valide
  $filename = implode('-', [$type, $bundle]) . '.csv';
  if (in_array($type, ['field_collection_item', 'paragraphs_item'])) {
    if (empty($field) || empty($bundle) || empty($entity_id)) {
      drupal_set_message(t("Yours variables is not available"), 'error');
      return $form;
    }
    $filename = implode('-', [$bundle, $field]) . '.csv';
  }

  $fields_info = _fields_info($bundle, $type, $field);
  $options_fields = _options_load_field($fields_info);
  $remove_none = array_shift($options_fields);
  $form['files']['csv_file_upload'] = [
    '#type' => 'managed_file',
    '#title' => t('Choose a file'),
    '#description' => '<a id="export" data-filename="' . $filename . '" download="' . $filename . '"> ' . t("Download CSV template: ") . '</a>
<span id="export-header" >' . implode(';', $options_fields) . '</span>
<br/><i>' . t('Use <b>,</b> for separate multiple values and "<b> - </b>" for separate multiple values in field') . '</i>
',
    '#required' => TRUE,
    '#upload_validators' => [
      'file_validate_extensions' => ['csv'],
    ],
  ];
  $form['files']['csv_delimiter'] = [
    '#type' => 'select',
    '#title' => t('Character used to separate'),
    '#options' => [
      ';' => ';',
      ',' => ',',
    ],
  ];

  $form['#attached']['js'] = [
    drupal_get_path('module', 'quick_data') . '/quick_data.js',
  ];

  $form['next'] = [
    '#type' => 'submit',
    '#value' => t('Next'),
  ];
  return $form;
}

function quick_data_import_form_submit($form, &$form_state) {
  // Handle page 1 submissions
  if ($form_state['clicked_button']['#id'] == 'edit-next') {
    $form_state['storage']['page_two'] = TRUE; // We set this to determine
    // which elements to display when the page reloads.
    // Values below in the $form_state['storage'] array are saved to carry forward to subsequent pages in the form.
    $form_state['storage']['page_one_values'] = $form_state['values'];
    $form_state["rebuild"] = TRUE;   // Added
  }
  // Handle page 2 submissions
  else {
    if ($form_state['storage']['type'] == 'paragraphs_item') {
      $info_field = field_info_field($form_state['storage']['field']);
      $info = field_info_instance($form_state['storage']['bundle'], $form_state['storage']['field'], $info_field["bundles"]["node"][0]);
      foreach ($info["settings"]["allowed_bundles"] as $bundle_first) {
        if ($bundle_first != -1) {
          $form_state['storage']['bundle_paragraphs'] = $bundle_first;
          break;
        }
      }
    }

    quick_data_import_mapping($form_state['storage'], $form_state["input"]["col"]);
    unset ($form_state['storage']); // This value must be unset for
    // redirection! This is because  $form_state['rebuild'] gets set to TRUE
    if (!empty($_GET['destination'])) {
      $form_state['redirect'] = $_GET['destination'];
    }
  }
}

// New function created to help make the code more manageable
function quick_data_import_process_csv($form, &$form_state) {

  $type = $form_state['storage']['type'];
  $bundle = $form_state['storage']['bundle'];
  $field = $form_state['storage']['field'];
  $entity_id = $form_state['storage']['entity_id'];
  $fields_info = _fields_info($bundle, $type, $field);
  $form_state['storage']['fields_info'] = $fields_info;
  $options_fields = _options_load_field($fields_info);
  $field = [
    '#type' => 'select',
    '#options' => $options_fields,
    '#empty_option' => t('- Select -'),
  ];
  $flip_fields = array_flip($options_fields);
  $csv_rows = _convert_csv_array($form_state["values"]["csv_file_upload"], $form_state["values"]["csv_delimiter"], 10);
  $header = [];
  $header_title = array_shift($csv_rows);
  foreach ($header_title as $col => $title_col) {
    $title_col = trim($title_col);
    $header[$col]["data"] = $field;
    $header[$col]["data"]["#title"] = $title_col;
    $header[$col]["data"]["#name"] = 'col[' . $col . ']';
    if (!empty($flip_fields[$title_col])) {
      $header[$col]["data"]["#value"] = $flip_fields[$title_col];
    }
  }

  $form['table'] = [
    '#theme' => 'table',
    "#title" => t("Mapping field with data"),
    '#header' => $header,
    '#rows' => $csv_rows,
    '#empty' => t('Empty'),
  ];
  $form['submit'] = [
    '#type' => 'submit',
    '#value' => t('Import'),
  ];
  return $form;
}

function _convert_csv_array($fid, $delimiter = ';', $limit = FALSE) {
  $file = file_load($fid);
  $filename = drupal_realpath($file->uri);
  $csv_rows = [];
  // Open the file for reading
  if (($h = fopen($filename, "r")) !== FALSE) {
    // Each line in the file is converted into an individual array that we call $data
    // The items of the array are comma separated
    $i = 0;
    while (($data = fgetcsv($h, 1000, $delimiter)) !== FALSE) {
      // Each individual array is being pushed into the nested array
      $csv_rows[] = $data;
      if ($limit && $i++ > $limit) {
        break;
      }
    }

    // Close the file
    fclose($h);
  }
  return $csv_rows;
}

function _table_info($table) {
  $schema = drupal_get_schema($table);
  $infos = [];
  $i = 0;
  foreach ($schema['fields'] as $name => $field) {
    $infos[$name] = [
      'field_name' => $name,
      'label' => !empty($field['description']) ? $field['description'] : $name,
      'type' => $field["type"],
      'columns' => ['value' => ['type' => $field["type"]]],
      'cardinality' => 1,
      'weight' => $i++,
    ];
  }
  return $infos;
}

function _fields_info($bundle, $type, $field = '') {
  if ($bundle == 'data') {
    return _table_info($type);
  }
  if (in_array($type, ['field_collection_item']) && !empty($field)) {
    $params = [
      'entity_type' => $type,
      'bundle' => $field,
    ];
    $fields = field_read_instances($params);
  }
  elseif (in_array($type, ['paragraphs_item']) && !empty($field)) {
    $info_field = field_info_field($field);
    $info = field_info_instance($bundle, $field, $info_field["bundles"]["node"][0]);
    foreach ($info["settings"]["allowed_bundles"] as $bundle_first) {
      if ($bundle_first != -1) {
        break;
      }
    }
    $params = [
      'entity_type' => $type,
      'bundle' => $bundle_first,
    ];
    $fields = field_read_instances($params);
  }
  else {
    $fields = field_info_instances($type, $bundle);
  }
  $infos = [];
  foreach ($fields as $field) {
    $field_info = field_info_field($field['field_name']);
    $module = $field["widget"]["module"];
    if (in_array($field["widget"]["module"], [
      'field_collection',
      'field_collection_table',
      'paragraphs',
      'paragraphs_table',
      'tablefield',
    ])) {
      continue;
    }
    $infos[$field['field_name']] = [
      'field_name' => $field['field_name'],
      'label' => $field['label'],
      'type' => $field_info["type"],
      'widget' => $field["widget"]["type"],
      'module' => $module,
      'columns' => $field_info["columns"],
      'cardinality' => !empty($field_info["cardinality"]) ? $field_info["cardinality"] : 1,
      'allowed_values' => !empty($field_info["settings"]["allowed_values"]) ? $field_info["settings"]["allowed_values"] : NULL,
      'weight' => $field["widget"]["weight"],
    ];
  }
  switch ($type) {
    case 'node':
      $infos['status'] = [
        'field_name' => 'status',
        'label' => t('Status'),
        'type' => 'status',
        'widget' => 'text_textfield',
        'module' => $bundle,
        'cardinality' => 1,
        'weight' => 1,
      ];
      $infos['title'] = [
        'field_name' => 'title',
        'label' => t('Title'),
        'type' => 'title',
        'widget' => 'text_textfield',
        'module' => $bundle,
        'cardinality' => 1,
        'weight' => 1,
      ];
      break;
    case 'user':
      $infos['status'] = [
        'field_name' => 'status',
        'label' => t('Status'),
        'type' => 'status',
        'widget' => 'text_textfield',
        'module' => $bundle,
        'cardinality' => 1,
        'weight' => 1,
      ];
      $infos['mail'] = [
        'field_name' => 'mail',
        'label' => t('Email '),
        'type' => 'mail',
        'widget' => 'text_textfield',
        'module' => $bundle,
        'cardinality' => 1,
        'weight' => 1,
      ];
      $infos['name'] = [
        'field_name' => 'name',
        'label' => t('Name'),
        'type' => 'name',
        'widget' => 'text_textfield',
        'module' => $bundle,
        'cardinality' => 1,
        'weight' => 1,
      ];
      break;
    case 'taxonomy':
      $infos['description'] = [
        'field_name' => 'description',
        'label' => t('description'),
        'type' => 'description',
        'widget' => 'text_textfield',
        'module' => $bundle,
        'cardinality' => 1,
        'weight' => 1,
      ];
      $infos['name'] = [
        'field_name' => 'name',
        'label' => t('Name'),
        'type' => 'name',
        'widget' => 'text_textfield',
        'module' => $bundle,
        'cardinality' => 1,
        'weight' => 1,
      ];
      break;
  }

  return $infos;
}

function _options_load_field($infos) {
  uasort($infos, 'drupal_sort_weight');
  $options = ['_none' => t('- None -')];
  foreach ($infos as $field_name => $info) {
    $options[$field_name] = $info['label'];
  }
  return $options;
}

function quick_data_import_mapping($store, $mapping) {
  $data = _convert_csv_array($store['page_one_values']['csv_file_upload'], $store['page_one_values']['csv_delimiter']);
  $header_title = array_shift($data);
  $id = $store['entity_id'];
  $host = FALSE;
  if (in_array($store['type'], [ 'field_collection_item','paragraphs_item']) && $id) {
    switch ($store['bundle']) {
      case 'node':
        $host = node_load($id);
        break;
      case 'user':
        $host = user_load($id);
        break;
      case 'field_collection_item':
        $host = field_collection_item_load($id);
        break;
      case 'taxonomy':
        $host = taxonomy_term_load($id);
        break;
    }
  }

  foreach ($data as $row_index => $row) {
    $entity = new stdClass();
    if ($store['type'] == 'field_collection_item') {
      $entity = entity_create('field_collection_item', ['field_name' => $store['field']]);
    }
    if ($store['type'] == 'paragraphs_item') {
      $entity = entity_create('paragraphs_item', [
        'field_name' => $store['field'],
        'bundle' => $store['bundle_paragraphs'],
      ]);
    }
    foreach ($row as $col => $value) {
      if (empty($value) || $mapping[$col] == '_none') {
        continue;
      }
      $field = $mapping[$col];
      $table_field[$field] = $value;
      quick_data_prepare_data($entity, $store['fields_info'][$field], $value);
    }
    switch ($store['type']) {
      case 'node':
        $entity->type = $store['bundle'];
        $entity->language = LANGUAGE_NONE;
        node_object_prepare($entity);
        try {
          node_save($entity);
        } catch (Exception $e) {
          drupal_set_message(implode(' ', $e->errorInfo), 'error');
        }
        break;
      case 'user':
        $entity->status = 1;
        $entity->pass = user_password();
        user_save($entity);
        break;
      case 'taxonomy':
        taxonomy_term_save($entity);
        break;
      case 'paragraphs_item':
      case 'field_collection_item':
        if ($host) {
          $entity->setHostEntity($store['bundle'], $host);
          $entity->save();
        }
        break;
      case 'data':
        if (!empty($table_field)) {
          $nid = db_insert($store['bundle'])
            ->fields($table_field)
            ->execute();
        }
        break;
    }
  }
}

function quick_data_prepare_data(&$entity, $field_info, $value) {
  $field = $field_info['field_name'];
  if (!empty($field_info["columns"])) {
    $key = array_key_first($field_info["columns"]);
    $type = $field_info["columns"][$key]["type"];
  }
  else {
    $key = $field_info["type"];
  }
  $delimiter = " - ";
  if ($field_info['cardinality'] == -1) {
    $values = explode(', ', $value);
  }
  else {
    $values = [$value];
  }
  foreach ($values as $index => $value) {
    switch ($field_info['type']) {
      case 'name':
        if ($field_info["module"] == 'name') {
          $checkName = explode($delimiter, $value);
          if (!empty($checkName[1])) {
            $entity->{$field}[LANGUAGE_NONE][$index]['family'] = $checkName[0];
            $entity->{$field}[LANGUAGE_NONE][$index]['given'] = $checkName[1];
            if (!empty($checkName[2])) {
              $entity->{$field}[LANGUAGE_NONE][$index]['title'] = $checkName[2];
            }
          }
          else {
            $entity->{$field}[LANGUAGE_NONE][$index]['family'] = $value;
          }
        }
        else {
          $entity->{$field_info['type']} = $value;
        }
        break;
      case 'status':
      case 'title':
      case 'mail':
        $entity->{$field_info['type']} = $value;
        break;

      case 'taxonomy_term_reference':
        if (!is_numeric($value)) {
          $term = taxonomy_get_term_by_name($value, $field_info["allowed_values"][0]["vocabulary"]);
          //create term if it is not exist
          if (!$term) {
            $term = new stdClass();
            $term->name = $value;
            $term->vid = taxonomy_vocabulary_machine_name_load($field_info["allowed_values"][0]["vocabulary"])->vid;;
            taxonomy_term_save($term);
          }
          if (!empty($term->tid)) {
            $value = $term->tid;
            $entity->{$field}[LANGUAGE_NONE][$index][$key] = $value;
          }
        }
        break;

      case 'user_reference':
        if (is_numeric($value)) {
          $entity->{$field}[LANGUAGE_NONE][$index][$key] = $value;
        }
        else {
          $user_load = user_load_by_name($value);
          if ($user_load) {
            $entity->{$field}[LANGUAGE_NONE][$index][$key] = $user_load->uid;
          }
        }
        break;

      case 'node_reference':
      case 'entityreference':
        if (is_numeric($value)) {
          $entity->{$field}[LANGUAGE_NONE][$index][$key] = $value;
        }
        else {
          $nodes = node_load_multiple(NULL, ["title" => $value]);
          if ($nodes) {
            $node_load = current($nodes);
            $entity->{$field}[LANGUAGE_NONE][$index][$key] = $node_load->nid;
          }
        }
        break;

      case 'list_integer':
      case 'list_float':
      case 'list_boolean':
      case 'list_text':
        if (empty($field_info["allowed_values"][$value])) {
          $flip_allowed_values = array_flip($field_info["allowed_values"]);
          $value = !empty($flip_allowed_values[$value]) ? $flip_allowed_values[$value] : '';
        }
        if (!empty($field_info["allowed_values"][$value])) {
          $entity->{$field}[LANGUAGE_NONE][$index][$key] = $value;
        }
        break;

      case 'number_integer':
      case 'number_float':
      case 'number_decimal':
        if (is_numeric($value)) {
          $entity->{$field}[LANGUAGE_NONE][$index][$key] = $value;
        }
        break;

      case 'date' :
      case 'datetime' :
        $value = str_replace('/', '-', $value);
        $checkEndDate = explode($delimiter, $value);
        if (!empty($checkEndDate[1])) {
          $entity->{$field}[LANGUAGE_NONE][$index]['value'] = date('Y-m-d H:i:s', strtotime($checkEndDate[0]));
          $entity->{$field}[LANGUAGE_NONE][$index]['value2'] = date('Y-m-d H:i:s', strtotime($checkEndDate[1]));
        }
        else {
          $entity->{$field}[LANGUAGE_NONE][$index][$key] = date('Y-m-d H:i:s', strtotime($value));
        }
        break;

      case 'datestamp' :
        $value = str_replace('/', '-', $value);
        $checkEndDate = explode($delimiter, $value);
        if (!empty($checkEndDate[1])) {
          $entity->{$field}[LANGUAGE_NONE][$index]['value'] = is_numeric($checkEndDate[0]) ? $checkEndDate[0] : strtotime($checkEndDate[0]);
          $entity->{$field}[LANGUAGE_NONE][$index]['value2'] = is_numeric($checkEndDate[1]) ? $checkEndDate[1] : strtotime($checkEndDate[1]);
        }
        else {
          $entity->{$field}[LANGUAGE_NONE][$index][$key] = is_numeric($value) ? $value : strtotime($value);
        }
        break;

      case 'double_field' :
        $checkSecond = explode($delimiter, $value);
        if (!empty($checkSecond[1])) {
          $entity->{$field}[LANGUAGE_NONE][$index]['first'] = $checkSecond[0];
          $entity->{$field}[LANGUAGE_NONE][$index]['second'] = $checkSecond[1];
        }
        else {
          $entity->{$field}[LANGUAGE_NONE][$index][$key] = $value;
        }
        break;

      case 'link' :
        $checkLink = explode($delimiter, $value);
        if (!empty($checkLink[1])) {
          $entity->{$field}[LANGUAGE_NONE][$index]['title'] = $checkLink[0];
          $entity->{$field}[LANGUAGE_NONE][$index]['url'] = $checkLink[1];
        }
        else {
          $entity->{$field}[LANGUAGE_NONE][$index][$key] = $value;
        }
        break;

      case "addressfield":
        $checkAddress = explode("<br />", nl2br(str_replace($delimiter, PHP_EOL, $value)));
        if (!empty($checkAddress[1])) {
          $separate = ["thoroughfare", "postal_code", "locality", "country"];
          foreach ($checkAddress as $indexAddress => $address) {
            $entity->{$field}[LANGUAGE_NONE][$index][$separate[$indexAddress]] = trim($address);
          }
          if (!empty($entity->{$field}[LANGUAGE_NONE][$index]["country"])) {
            $entity->{$field}[LANGUAGE_NONE][$index]["country"] = variable_get('site_default_country');
          }
        }
        else {
          $entity->{$field}[LANGUAGE_NONE][$index]["thoroughfare"] = $value;
        }
        break;

      default:
        $entity->{$field}[LANGUAGE_NONE][$index][$key] = $value;
        break;
    }
  }
}
